@@ -1,19 +1,93 @@
-NbClust <-function(data, diss=NULL, distance ="euclidean", min.nc=2, max.nc=15, method =NULL, index = "all", alphaBeale = 0.1)
+NbClust <-function(data = NULL, diss=NULL, distance ="euclidean", min.nc=2, max.nc=15, method =NULL, index = "all", alphaBeale = 0.1)
 {
     
     x<-0
     min_nc <- min.nc
     max_nc <- max.nc
-    jeu1 <- as.matrix(data)
-    numberObsBefore <- dim(jeu1)[1]
-    jeu <- na.omit(jeu1) # returns the object with incomplete cases removed 
-    nn <- numberObsAfter <- dim(jeu)[1]
-    pp <- dim(jeu)[2]    
-    TT <- t(jeu)%*%jeu   
-    sizeEigenTT <- length(eigen(TT)$value)
-    eigenValues <- eigen(TT/(nn-1))$value
+    
+    if(is.null(method))    
+      stop("method is NULL")
+    method <- pmatch(method, c("ward.D2", "single", "complete", "average", 
+                               "mcquitty", "median", "centroid", "kmeans","ward.D"))
+    
+        
+    indice <- pmatch(index, c("kl","ch","hartigan","ccc","scott","marriot","trcovw","tracew","friedman",
+                              "rubin","cindex","db","silhouette","duda","pseudot2","beale","ratkowsky","ball",
+                              "ptbiserial","gap", "frey", "mcclain",  "gamma", "gplus", "tau", "dunn", 
+                              "hubert", "sdindex", "dindex", "sdbw", "all","alllong"))
+    if (is.na(indice))
+      stop("invalid clustering index")
+    
+    if (indice == -1)
+      stop("ambiguous index")
+    
+    if ((indice == 3)|| (indice == 5)|| (indice == 6)|| (indice == 7)|| (indice == 8)|| (indice == 9)|| (indice == 10)|| (indice == 11)|| (indice == 18)|| (indice == 27)|| (indice == 29)|| (indice == 31)|| (indice == 32))
+    { 
+      if((max.nc-min.nc)<2)
+        stop("The difference between the minimum and the maximum number of clusters must be at least equal to 2")
+    }
+    
+    
+    
+    if(is.null(data))
+    {
+       
+      if(method==8)
+      {
+        stop("\n","method = kmeans, data matrix is needed")
+      }
+      else
+      {  
+        if ((indice == 1 )|| (indice == 2)|| (indice == 3 )|| (indice == 4 )|| (indice == 5)|| (indice == 6)|| (indice == 7)|| (indice == 8)|| (indice == 9)|| (indice == 10)|| (indice == 12)|| (indice == 14)|| (indice == 15)|| (indice == 16)|| (indice == 17)|| (indice == 18)
+          || (indice == 19)|| (indice == 20)|| (indice == 23)|| (indice == 24)|| (indice == 25) || (indice == 27)|| (indice == 28)
+          || (indice == 29) || (indice == 30)|| (indice == 31) || (indice == 32))
+          stop("\n","Data matrix is needed. Only frey, mcclain, cindex, sihouette and dunn can be computed.", "\n")
+       
+        if(is.null(diss))
+          stop("data matrix and dissimilarity matrix are both null")
+        else  
+          cat("\n","Only frey, mcclain, cindex, sihouette and dunn can be computed. To compute the other indices, data matrix is needed","\n") 
+      }
+    }
+    
+    else
+    {
+      jeu1 <- as.matrix(data)
+      numberObsBefore <- dim(jeu1)[1]
+      jeu <- na.omit(jeu1) # returns the object with incomplete cases removed 
+      nn <- numberObsAfter <- dim(jeu)[1]
+      pp <- dim(jeu)[2]    
+      TT <- t(jeu)%*%jeu   
+      sizeEigenTT <- length(eigen(TT)$value)
+      eigenValues <- eigen(TT/(nn-1))$value
+      
+      # Only for indices using vv : CCC, Scott, marriot, tracecovw, tracew, friedman, rubin
+      
+      if (any(indice == 4) || (indice == 5) || (indice == 6) || (indice == 7) || (indice == 8) || (indice == 9) || (indice == 10) || (indice == 31) || (indice == 32))
+      {
+        for (i in 1:sizeEigenTT) 
+        {
+          if (eigenValues[i] < 0) {
+            #cat(paste("There are only", numberObsAfter,"nonmissing observations out of a possible", numberObsBefore ,"observations."))
+            stop("The TSS matrix is indefinite. There must be too many missing values. The index cannot be calculated.")
+          } 
+        }
+        s1 <- sqrt(eigenValues)
+        ss <- rep(1,sizeEigenTT)
+        for (i in 1:sizeEigenTT) 
+        {
+          if (s1[i]!=0) 
+            ss[i]=s1[i]
+        }
+        vv <- prod(ss)  
+      } 
+            
+    }
     
      
+    
+  
+      
   #&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&#
   #                                                                                                                      #
   #                                              Distances                                                               #
@@ -40,19 +114,19 @@ if(is.null(diss))
     if (distanceM == 2) 
       {
     		md <- dist(jeu, method="maximum")	
-	}
+	    }
     if (distanceM == 3) 
       {
     		md <- dist(jeu, method="manhattan")	
-	}
+	    }
     if (distanceM == 4) 
       {
     		md <- dist(jeu, method="canberra")	
-	}
+	    }
     if (distanceM == 5) 
       {
     		md <- dist(jeu, method="binary")	
-	}
+	    }
     if (distanceM == 6) 
      {
     		md <- dist(jeu, method="minkowski")	
@@ -67,7 +141,7 @@ if(is.null(diss))
 if(!is.null(diss))
 {
   if((distanceM==1)||(distanceM==2)|| (distanceM==3)|| (distanceM==4)|| (distanceM==5)|| (distanceM==6))
-stop("dissimilarity matrix and distance are both not null")
+    stop("dissimilarity matrix and distance are both not null")
   else
     md <- diss
 }
@@ -80,11 +154,7 @@ stop("dissimilarity matrix and distance are both not null")
     #                                                                                                                      #
     #&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&#
   
-if(is.null(method))    
-    stop("method is NULL")
-  method <- pmatch(method, c("ward.D2", "single", "complete", "average", 
-		"mcquitty", "median", "centroid", "kmeans","ward.D"))
-
+   
     res <- array(0, c(max_nc-min_nc+1,30))
     x_axis <- min_nc:max_nc
     resCritical <- array(0, c(max_nc-min_nc+1,4))
@@ -1207,7 +1277,7 @@ Indice.Gap <- function (x, clall, reference.distribution = "unif", B = 10,
     if (sum(c("centroids", "medoids") == centrotypes) == 0) 
         stop("Wrong centrotypes argument")
     if ("medoids" == centrotypes && is.null(d)) 
-        stop("For argument centrotypes = 'medoids' d cannot be null")
+        stop("For argument centrotypes = 'medoids' d can not be null")
     if (!is.null(d)) {
         if (!is.matrix(d)) {
             d <- as.matrix(d)
@@ -1317,43 +1387,6 @@ Indice.Gap <- function (x, clall, reference.distribution = "unif", B = 10,
 ################
 
 
- indice <- pmatch(index, c("kl","ch","hartigan","ccc","scott","marriot","trcovw","tracew","friedman",
-		"rubin","cindex","db","silhouette","duda","pseudot2","beale","ratkowsky","ball",
-		"ptbiserial","gap", "frey", "mcclain",  "gamma", "gplus", "tau", "dunn", 
-    "hubert", "sdindex", "dindex", "sdbw", "all","alllong"))
-    if (is.na(indice))
-	     stop("invalid clustering index")
-    if (indice == -1)
-      	stop("ambiguous index")
- 
-    if ((indice == 3)|| (indice == 5)|| (indice == 6)|| (indice == 7)|| (indice == 8)|| (indice == 9)|| (indice == 10)|| (indice == 11)|| (indice == 18)|| (indice == 27)|| (indice == 29)|| (indice == 31)|| (indice == 32))
-    { 
-      if((max.nc-min.nc)<2)
-        stop("The difference between the minimum and the maximum number of clusters must be at least equal to 2")
-    }
-
-# Only for indices using vv : CCC, Scott, marriot, tracecovw, tracew, friedman, rubin
-
-if (any(indice == 4) || (indice == 5) || (indice == 6) || (indice == 7) || (indice == 8) || (indice == 9) || (indice == 10) || (indice == 31) || (indice == 32))
-{
-  for (i in 1:sizeEigenTT) 
-  {
-    if (eigenValues[i] < 0) {
-      cat(paste("There are only", numberObsAfter,"nonmissing observations out of a possible", numberObsBefore ,"observations."))
-      stop("The TSS matrix is indefinite. There must be too many missing values. The index cannot be calculated.")
-    } 
-  }
-  s1 <- sqrt(eigenValues)
-  ss <- rep(1,sizeEigenTT)
-  for (i in 1:sizeEigenTT) 
-  {
-    if (s1[i]!=0) 
-      ss[i]=s1[i]
-  }
-  vv <- prod(ss)  
-} 
-    
-    
  for (nc in min_nc:max_nc)
  {  
       
@@ -2237,15 +2270,15 @@ if (any(indice == 4) || (indice == 5) || (indice == 6) || (indice == 7) || (indi
   res<-round(res, digits=4)
   resCritical<-round(resCritical, digits=4)
 
-  if (numberObsAfter != numberObsBefore) 
-  {
-     cat(paste(numberObsAfter,"observations were used out of", numberObsBefore ,"possible observations due to missing values."))
-  }
+#  if (numberObsAfter != numberObsBefore) 
+#  {
+#     cat(paste(numberObsAfter,"observations were used out of", numberObsBefore ,"possible observations due to missing values."))
+#  }
   
-  if (numberObsAfter == numberObsBefore) 
-  {
-     cat(paste("All", numberObsAfter,"observations were used.", "\n", "\n"))
-  }
+#  if (numberObsAfter == numberObsBefore) 
+#  {
+#     cat(paste("All", numberObsAfter,"observations were used.", "\n", "\n"))
+#  }
   
   
     
